package io.nisum.designpattern.department.controller;

import io.nisum.designpattern.department.dto.DepartmentDto;
import io.nisum.designpattern.department.services.DepartmentService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@Slf4j
@RestController
@RequestMapping("/departments")
public class DepartmentController {

    @Autowired
    DepartmentService departmentService;

    @PostMapping
    public Mono<DepartmentDto> save( @RequestBody Mono<DepartmentDto> department ) {
        log.info("Department add: {}", department);
        return departmentService.save(department);
    }

    @GetMapping("/{id}")
    public Mono<DepartmentDto> findById(@PathVariable("id") long id) {
        log.info("Department find: id={}", id);
        return departmentService.findById(id);
    }

    @GetMapping
    public Flux<DepartmentDto> findAll() {
        log.info("Department find");
        return departmentService.findAll();
    }

    @GetMapping("/organization/{organizationId}")
    public Flux<DepartmentDto> findByOrganization(@PathVariable("organizationId") long organizationId) {
        log.info("Department find: organizationId={}", organizationId);
        return departmentService.findByOrganization(organizationId);
    }

}
