package io.nisum.designpattern.department.services.impl;

import io.nisum.designpattern.department.dto.DepartmentDto;
import io.nisum.designpattern.department.mapper.MapperModel;
import io.nisum.designpattern.department.repo.DepartmentRepository;
import io.nisum.designpattern.department.services.DepartmentService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@Service
public class DepartmentServiceImpl implements DepartmentService {

    @Autowired
    DepartmentRepository repository;

    @Override
    public Mono<DepartmentDto> save(Mono<DepartmentDto> departmentDtoMono) {
        return departmentDtoMono
                .map(MapperModel::mapToEntity)
                .flatMap(repository::save)
                .map(MapperModel::mapToDto);
    }

    @Override
    public Mono<DepartmentDto> findById(long id) {
        return repository
                .findById(id)
                .map(MapperModel::mapToDto);
    }

    @Override
    public Flux<DepartmentDto> findAll() {
        return repository
                .findAll()
                .map(MapperModel::mapToDto);
    }

    @Override
    public Flux<DepartmentDto> findByOrganization(long organizationId) {
        return repository
                .findByOrganizationId(organizationId)
                .map(MapperModel::mapToDto);
    }
}
