package io.nisum.designpattern.employee.controller;

import io.nisum.designpattern.employee.dto.EmployeeDto;
import io.nisum.designpattern.employee.dto.EmployeePatchDto;
import io.nisum.designpattern.employee.service.EmployeeService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@Slf4j
@RestController
@RequestMapping("/employees")
public class EmployeeController {

    @Autowired
    EmployeeService employeeService;

    @GetMapping
    @ResponseStatus(HttpStatus.OK)
    public Flux<EmployeeDto> findAll() {
        log.info("Employee find");
        return employeeService.findAll();
    }

    @GetMapping("/{id}")
    public Mono<EmployeeDto> findById( @PathVariable("id") long id ) {
        log.info("Employee find: id={}", id);
        return employeeService.findById(id);
    }

    @GetMapping("/name/{name}")
    public Mono<EmployeeDto> findByName( @PathVariable("name") String name ) {
        log.info("Employee find: name={}", name);
        return employeeService.findByName(name);
    }

    @GetMapping("/department/{depId}")
    public Flux<EmployeeDto> findByDepartment( @PathVariable("depId") long departmentId ) {
        log.info("Employee find: departmentId={}", departmentId);
        return employeeService.findByDepartment(departmentId);
    }

    @GetMapping("/organization/{orgId}")
    public Flux<EmployeeDto> findByOrganization( @PathVariable("orgId") long organizationId ) {
        log.info("Employee find: organizationId={}", organizationId);
        return employeeService.findByOrganization(organizationId);
    }

    @PostMapping
    public Mono<EmployeeDto> save( @RequestBody EmployeeDto employeeDto ) {
        log.info("Employee save: employee={}", employeeDto);
        return employeeService.save( Mono.just(employeeDto) );
    }

    @PutMapping("/{id}")
    public Mono<EmployeeDto> update( @PathVariable("id") long id, @RequestBody EmployeeDto employeeDto ) {
        log.info("Employee update: id={}, employee={}", id, employeeDto);
        return employeeService.update( id, Mono.just(employeeDto) );
    }

    @DeleteMapping("/{id}")
    public Mono<Void> delete( @PathVariable("id") long id ) {
        log.info("Employee delete: id={}", id);
        return employeeService.delete( id );
    }

}